# -*- coding: utf-8 -*-
"""Test configuration for pytest"""

import pytest
from tests.fixtures.mock_session import mock_session, make_mock_response
from tests.fixtures.mock_streaming_client import mock_streaming_client, mock_generator
